function ideal = ideal_threshold(nb_obs,dim,varargin)
%
% This function computes the ideal threshold, as defined in the paper ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% IN : 
% 
%	'nb_obs' : number of observations
%	'dim'    : dimension 
%	
% followed by a list of arguments of the form 'property',value
%      (if not given the default is used)
%
%	'data_type' : which kind of data are to be generated ?
%		currently available:
%		'gauss_texture' (default)
%
%	'filter' : option for the 'gauss_texture' case (shape of the filter)
%		default 'exp(d^2)'
%
%	'bandwidth' : option for the 'gauss_texture' case (width of the filter)
%		default 30
%
%   'mean' : different kinds of mean vector are currently available
%       'zero' (default) : mu=0
%       'linear'        : setting of Section 4.3 : half of the means are
%           zero ; the other half are increasing linearly from
%           (5/16)*(2 sided Bonferroni threshold) to 20*(2 sided Bonferroni
%           threshold) (assuming level=0.05)
%       'exponential'  : setting of Section 4.4
%
%   'dynamic_range' : option for the 'exponential' mean setting (default
%   20)
%
%   'sigma' : value of the (constant) standard deviation (default 1)
%
%
%   'nb_simu' : number of data samples to be generated for estimating the
%   ideal threshold (default 1000)
%
%	'type' : type of experiment (function phi);
%                currently available:
%                   'sup_one_sided'
%                   'sup_two_sided' (default)
%
%   'level' : confidence level of the threshold
%           default 0.05
%
%
%  OUT : 'Y' is a dim*nb_obs data matrix 
%

opt = propertylist2struct(varargin{:});

[opt, isdefault] = set_defaults(opt, 'filter','exp(d^2)', ...
                    'bandwidth',30,...
				    'data_type','gauss_texture',...
                    'sigma',1,...
                    'dynamic_range',20,...
                    'nb_simu',1000,...
                    'type','sup_two_sided',...
                    'level',0.05,...
                    'mean','zero'); 
                
ideal_v=zeros(1,opt.nb_simu);

switch opt.data_type
    case 'gauss_texture'
        % data are Gaussian and centered, so that the empirical mean of Y
        % has the same distribution as (Y^1)/sqrt(n)

        for i_sample=1:opt.nb_simu

            Empmean=gener_data(1,dim,'data_type',opt.data_type,'filter',opt.filter,'bandwidth',opt.bandwidth,'mean',opt.mean,'sigma',opt.sigma)/sqrt(nb_obs);                
            mu_v=mean_vector(opt.mean,dim,'dynamic_range',opt.dynamic_range,'nb_obs',nb_obs,'sigma',opt.sigma);
            switch (opt.type)
                case 'sup_one_sided'
                    ideal_v(i_sample)=max(max(Empmean(mu_v <= 0),0));
                case 'sup_two_sided'
                    ideal_v(i_sample)=max(abs(Empmean(mu_v == 0)));
                otherwise
                    ideal_v(i_sample)=NaN;
            end;%switch (opt.type)

        end%for i_sample=1:nb_simu
    otherwise
        ideal_v=NaN*ideal_v;
end

ideal_v=-sort(-ideal_v);
ideal=ideal_v(floor(opt.level * opt.nb_simu));

    