function mu = mean_vector(mean,dim,varargin)
%
% This function computes the mean vector for the experiments defined in the paper ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% IN : 
% 
%   'mean' : different kinds of mean vector are currently available
%       'zero' (default) : mu=0
%       'linear'        : setting of Section 4.3 : half of the means are
%           zero ; the other half are increasing linearly from
%           (5/16)*(2 sided Bonferroni threshold) to 20*(2 sided Bonferroni
%           threshold) (assuming level=0.05)
%       'exponential'  : setting of Section 4.4
%
%	'dim'    : dimension 
%
% followed by a list of arguments of the form 'property',value
%      (if not given the default is used)
%
%   'dynamic_range' : option for the 'exponential' mean setting (default
%   20)
%
%	'nb_obs' : number of observations (default 1000)
%   'sigma' : value of the (constant) standard deviation (default 1)
%
%  OUT : 'mu' is a (row) vector of size dim
%

opt = propertylist2struct(varargin{:});

[opt, isdefault] = set_defaults(opt, 'nb_obs',1000,...
                    'sigma',1,...
                    'dynamic_range',20); 

switch (mean)
    case 'zero'
        mu=zeros(1,dim);
    case 'linear'
        ed = floor(sqrt(dim));
        if ( dim ~= ed*ed)
            error('Dimension must be a square.');
        end

        bon=-opt.sigma*gausinvcdf(0.05/(2*dim)) / sqrt(opt.nb_obs);
        mu = repmat( max(0,1 - (((0:(ed-1))/(ed/2)) ) )*20*bon , [1 ed]);
    case 'exponential'
        ed = floor(sqrt(dim));
        if ( dim ~= ed*ed)
            error('Dimension must be a square.');
        end
        bon=-opt.sigma*gausinvcdf(0.05/(2*dim)) / sqrt(opt.nb_obs);
        mu=0.5*bon*exp((1 - (2*(0:(dim-1))/dim)) * opt.dynamic_range * log(10)/10 );
        mu((1+floor(dim/2)):dim)=zeros(1,(dim-floor(dim/2)));
    otherwise
        mu=NaN*ones(1,dim);
end%switch (opt.mean)
