function result=thresholds(Ymat,varargin)
%
% This function computes the thresholds defined in the paper ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% result=thresholds(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma,'weights',weights,'thresholds_list',thresholds_list,'nb_mcmc',nb_mcmc,...
%   'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);
% 
%
% IN : 
% 
%	'Ymat' : data matrix of size K x n. The columns are supposed to be iid. The multiple test looks for the rows having a mean different from zero.
%
% followed by a list of arguments of the form 'property',value
%      (if not given the default is used)
%
%	'level' : confidence level of the procedure
%                default 0,05
%	'type' : type of experiment (function phi);
%                currently available:
%                   'sup_one_sided'
%                   'sup_two_sided' (default)
%	'data_law' : assumption on the distribution of the data: 
%                currently available:
%                   'Gaussian' (default)
%                   'symmetric'
%	'sigma' : (known or estimated) value of phi(sigma)
%		default: estimates it with the function sigma_estim 
%	'weights' : distribution of the weights used for computing the concentration thresholds
%		currently available:
%		'Rad'    : Rad(1/2) weights (default)
%		'Rad(p)' : Bernoulli (p) weights
%		'Efr'    : Efron's bootstrap (n) weights
%		'Efr(M)' : Efron's bootstrap (M) weights
%		'Rho'    : Random hold-out (n/2) weights
%		'Rho(q)' : Random hold-out (q) weights
%		'Loo'    : Leave-one-out weights
%
%	'thresholds_list' : list of the thresholds to be computed 
%		largest list available : {'bonf','conc','conc_bonf','quant_bonf','quant_conc','single','quant','quant_uncent'}
%		default {'bonf','conc','conc_bonf','quant_bonf','quant_conc','single','quant','quant_uncent'}
%
%	'nb_mcmc' : number of resamplings for each experiment
%		default 1000
%	'deltacb' : parameter delta used in threshold min(conc,bonf)
%		default 0.1 
%	'deltaq' : parameter delta used in the quantile threshold
%		default 0.1 
%	'alpha0_mult' : will be multiplied by the level to compute alpha_0 used in the quantile threshold
%		default 0.9
%	'AW' : value of the constant A_W (default = compute it with A_W.m)
%	'BW' : value of the constant B_W (default = compute it with B_W.m)
%	'CW' : value of the constant C_W (default = compute it with C_W.m)
%	'DW' : value of the constant D_W (default = compute it with D_W.m)
%	'gamma' : value of the constant gamma_n(alpha_0 delta) (default = compute it from the other parameters)
%
%  OUT : result is a struct variable with fields:
%
%	'parameters' : store all the above parameters for the record
%	'bonf'       : Bonferroni threshold
%	'single'     : single test threshold
%	'conc'       : concentration threshold
%	'conc_bonf'  : min(conc,bonf) threshold
%	'quant_conc' : quantile threshold with 'conc' correction term
%	'quant_bonf' : quantile threshold with 'bonf' correction term
%	'quant'      : quantile threshold (without correction term) = 'centered symmetrization quantile'
%	'quant_uncent' : uncentered quantile threshold 'classical symmetrization quantile'
%

opt = propertylist2struct(varargin{:});

[opt, isdefault] = set_defaults(opt, 'deltaq', 0.1, ...
				    'alpha0_mult', 0.9,...
				    'deltacb', 0.1,...
				    'level', 0.05,...
				     'type','sup_two_sided',...
				    'sigma',NaN,...
				    'nb_mcmc', 1000,...
				     'weights','Rad',...
			             'data_law','Gaussian',...
				     'AW',NaN,...
				     'BW',NaN,...
				     'CW',NaN,...
				     'DW',NaN,...
				     'gamma',NaN,...
				     'thresholds_list',{'bonf','conc','conc_bonf','quant_bonf','quant_conc','single','quant','quant_uncent'});

%%% Which thresholds should be computed ?
%%% Bonferroni package
z_bonf = max([ max(strcmp(opt.thresholds_list,'bonf')) max(strcmp(opt.thresholds_list,'quant_bonf')) max(strcmp(opt.thresholds_list,'single')) max(strcmp(opt.thresholds_list,'conc_bonf')) ]);
%%% Concentration package
z_conc = max([ max(strcmp(opt.thresholds_list,'conc')) max(strcmp(opt.thresholds_list,'quant_conc')) max(strcmp(opt.thresholds_list,'conc_bonf')) ]);
%%% Centered quantiles package
z_quant = max([ max(strcmp(opt.thresholds_list,'quant_bonf')) max(strcmp(opt.thresholds_list,'quant')) max(strcmp(opt.thresholds_list,'quant_conc')) ]);
%%% Uncentered quantiles package
z_quant_uncent = max(strcmp(opt.thresholds_list,'quant_uncent'));
%%% Resampling package
z_resampl = max([z_quant z_quant_uncent z_conc]);

if (isnan(opt.sigma) && z_bonf)
	opt.sigma=sigma_estim(Ymat,opt.type);
end

[opt.dim opt.nb_obs]=size(Ymat);

if (isnan(opt.AW) && strcmp(opt.data_law,'symmetric') && z_conc)
	opt.AW=A_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.BW) && strcmp(opt.data_law,'Gaussian') && z_conc)
	opt.BW=B_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.CW) && strcmp(opt.data_law,'Gaussian') && z_conc)
	opt.CW=C_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.DW) && strcmp(opt.data_law,'symmetric') && z_conc)
	opt.DW=D_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.gamma) && z_quant)
	opt.gamma = 1-2*binoinv(opt.deltaq*opt.level*opt.alpha0_mult/2,opt.nb_obs,0.5)/(opt.nb_obs);
end

result.parameters = opt; %store the used parameters in the results,
                         %that helps later


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Computes the centered data matrix
Empmean = mean(Ymat,2);
Centered_dat = Ymat - repmat(Empmean,[1 opt.nb_obs]);

if z_resampl
    %%% Generates a matrix of weights
    if (z_conc || (strcmp(opt.weights,'Rad')))
        Wmat = gener_Wmat(opt.nb_obs,opt.weights,opt.nb_mcmc);
    end%if (z_conc || (strcmp(opt.weights,'Rad')))

    if (z_conc || (z_quant  && (strcmp(opt.weights,'Rad'))))
        %%% Compute the nb_mcmc realization of the resampled empirical process 
        R_process = Centered_dat*Wmat/opt.nb_obs;
        % R_process is now a dim x nb_mcmc table containing the means of the
        % recentered, reweighted data for nb_mcmc different draws of the
        % weights

        %%% the variable phi_rprocess contains
        % phi(n^{-1} sum_1=1}^n W_i Y^i_k) for nb_mcmc draws of the weights
        switch (opt.type)
            case 'sup_one_sided'
                phi_rprocess=max(max(R_process),0);
            case 'sup_two_sided'
                phi_rprocess=max(abs(R_process));
        end;%switch (opt.type)
    end%if (z_conc || (z_quant  && (strcmp(opt.weights,'Rad'))))

    if z_conc
        % Resampled expectation
        R_expectation = (mean(phi_rprocess))/opt.BW;
    end%if z_conc
    
    if z_quant
        %%% Quantiles: needs to compute again the resampled empirical process if weights were not 'Rad'
        if strcmp(opt.weights,'Rad')
            phi_radprocess=phi_rprocess;
        else
            WmatRad = gener_Wmat(opt.nb_obs,'Rad',opt.nb_mcmc);
            Rad_process = Centered_dat*WmatRad/opt.nb_obs;
            switch (opt.type)
                case 'sup_one_sided'
                    phi_radprocess=max(max(Rad_process),0);
                case 'sup_two_sided'
                    phi_radprocess=max(abs(Rad_process));
            end;%switch (opt.type)
        end%weights=='Rad'

        % Resampled quantile at level alpha (needs the weights to be Rademacher weights)
        spr=-sort(-phi_radprocess);
        R_quantile_alpha = spr(floor(opt.level * numel(spr)));
        % Resampled quantile at level alpha_0 * (1 - delta)
        R_quantile_alphaetc = spr(floor((opt.level) * (opt.alpha0_mult) * (1 - opt.deltaq) * numel(spr))); 
    end%if z_quant

    if z_quant_uncent
        %%% Uncentered resampled empirical process
        if strcmp(opt.weights,'Rad')
            if (z_quant || z_conc)
                R_process_uncent = R_process + Empmean*mean(Wmat,1); % makes use of the previous computations
            else
                R_process_uncent = Ymat*Wmat/opt.nb_obs; % same matrix but longer computation time
            end%if (z_quant || z_conc)
        else
            if z_quant
                R_process_uncent = Rad_process + Empmean*mean(WmatRad,1); % makes use of the previous computations
            else
                R_process_uncent = Ymat*WmatRad/opt.nb_obs; % same matrix but longer computation time
            end%if z_quant
        end%strcmp(opt.weights,'Rad')

        switch (opt.type)
            case 'sup_one_sided'
                phi_radprocessun=max(max(R_process_uncent),0);
            case 'sup_two_sided'
                phi_radprocessun=max(abs(R_process_uncent));
        end;%switch (opt.type)

        spru=-sort(-phi_radprocessun);
        R_quantile_uncent_alpha = spru(floor(opt.level * numel(spru)));
    end%z_quant_uncent

%clear spr,R_process,phi_rprocess ;
end%if z_resampl

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Compute all the thresholds
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Bonferroni threshold, single threshold, part of the min(conc,bonf) threshold and remainder term for the quant+bonf threshold
%%%
if (z_bonf && strcmp(opt.data_law,'Gaussian'))
	switch (opt.type)
	case 'sup_one_sided'
		result.bonf = -opt.sigma*gausinvcdf(opt.level/opt.dim)/sqrt(opt.nb_obs); 
		result.single = -opt.sigma*gausinvcdf(opt.level)/sqrt(opt.nb_obs); 
			% the modified Bonferroni bound to use in the min(bonf,conc)
		minbonf_bound = -opt.sigma*gausinvcdf(opt.level*(1-opt.deltacb)/opt.dim)/sqrt(opt.nb_obs);
% 			the remainder term for the quant+bonf threshold 
		result.remain_bonf = -(opt.gamma) * (opt.sigma) * gausinvcdf(opt.level*(1-opt.alpha0_mult)/opt.dim) / sqrt(opt.nb_obs);
	case 'sup_two_sided'
		result.bonf = -opt.sigma * gausinvcdf(opt.level/(2*opt.dim)) / sqrt(opt.nb_obs); 
		result.single = -opt.sigma * gausinvcdf(opt.level/2) / sqrt(opt.nb_obs); 
			% the modified Bonferroni bound to use in the min(bonf,conc)
		minbonf_bound = -opt.sigma * gausinvcdf(opt.level*(1-opt.deltacb)/(2*opt.dim)) / sqrt(opt.nb_obs);
% 			the remainder term for the quant+bonf threshold 
		result.remain_bonf = -(opt.gamma) * (opt.sigma) * gausinvcdf(opt.level*(1-opt.alpha0_mult)/(2*opt.dim)) / sqrt(opt.nb_obs);
	end;%switch (opt.type)
else
	result.bonf=NaN;
	minbonf_bound=NaN;
end%(z_bonf && strcmp(opt.data_law,'Gaussian'))

if max(strcmp(opt.thresholds_list,'conc'))
    %concentration threshold
    result.conc = R_expectation - opt.sigma *gausinvcdf(opt.level/2) * ((1/(opt.nb_obs*opt.BW))+(1/sqrt(opt.nb_obs))) ;
    % the remainder term for the quant+conc threshold 
    result.remain_conc = (opt.gamma)*(R_expectation - opt.sigma *gausinvcdf(opt.level*(1-opt.alpha0_mult)/2) * ((1/((opt.nb_obs)*opt.BW))+(1/sqrt(opt.nb_obs)))) ; 
end%if max(strcmp(opt.thresholds_list,'conc'))

if max(strcmp(opt.thresholds_list,'conc_bonf'))
    % min(conc,bonf) threshold
    result.conc_bonf= min( minbonf_bound , R_expectation - (opt.sigma/sqrt(opt.nb_obs))*gausinvcdf(opt.level * (1 - opt.deltacb)/2) - (opt.sigma/((opt.nb_obs)*opt.BW))*gausinvcdf(opt.level * opt.deltacb/2)) ;
end%if max(strcmp(opt.thresholds_list,'conc_bonf'))

if max(strcmp(opt.thresholds_list,'quant_conc'))
    %quantile threshold with 'conc' correction term
    result.quant_conc=R_quantile_alphaetc + result.remain_conc;
end%if max(strcmp(opt.thresholds_list,'quant_conc'))

if max(strcmp(opt.thresholds_list,'quant_bonf'))
    %quantile threshold with 'bonf' correction term
    result.quant_bonf=R_quantile_alphaetc + result.remain_bonf;
end%if max(strcmp(opt.thresholds_list,'quant_bonf'))

if max(strcmp(opt.thresholds_list,'quant'))
    %quantile threshold (without correction term) = 'centered symmetrization quantile'
    result.quant = R_quantile_alpha;
end%if max(strcmp(opt.thresholds_list,'quant'))

if max(strcmp(opt.thresholds_list,'quant_uncent'))
    %uncentered quantile threshold ('classical symmetrization quantile')
    result.quant_uncent=R_quantile_uncent_alpha;
end%if max(strcmp(opt.thresholds_list,'quant_uncent'))
